/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

ASSIGNMENT * algorithm_new( UC_CURVE * uc_curve,
			    LOSS_PROFILE * profile,
			    CONFIG_DATA * config)
{
  int i, N;
  int iteration;
  double cost_fraction = 1.0;

  double block_cost;
  double block_util;

  double this_cost;
  double this_util;

  double best_slope;
  double best_cost;
  double best_util;
  int	 best_index;

  double current_slope;
  
  double used_cost;
  double used_util;

  ASSIGNMENT * assignment;

  UC_ENTRY * *	uc_point;
  
  UC_ENTRY *	ucp;
  UC_ENTRY *	head;

  double *	utilities;

  N = profile->number_descriptions;

  uc_point = malloc( sizeof(UC_ENTRY *) * (N+1) );
  check_block( uc_point );
  utilities = malloc( sizeof(double) * (N+1) );
  check_block( utilities );
  for( i = 0; i <= N; i++ )
    utilities[i] = 0.0;

  assignment = alloc_assignment( config );

  head = uc_curve->convex_head;
  for( i = 0; i <= N; i++ )
    {
      uc_point[i] = head;
      this_util = head->utility;
      utilities[i] = this_util;
    }

  used_cost = head->cost;
  used_util = head->utility;
  iteration = 0;

  while( used_cost < config->cost_to_use )
    {
      iteration++;
      best_index = -1;
      best_slope = -1;
      best_cost	 = 0.0;
      best_util	 = 0.0;
      block_cost = 0.0;
      block_util = 0.0;

      for( i = N; i > 0; i-- )
	{
	  double rs_code_cost;

	  ucp = uc_point[i];
	  rs_code_cost = ((double) N / (double) i);
	  if ( i != N )
	    rs_code_cost -= ((double) N / (double) (i+1));

	  if ( ucp->next )
	    {
	      this_cost = ucp->next->cost - ucp->cost;
	      this_cost *= rs_code_cost;
	      
	      this_util = ucp->next->utility - ucp->utility;
	      this_util *= profile->receive_PMF[i];
	    }
	  else
	    {
	      this_cost = 10000.0;
	      this_util = -1.0;
	    }
	      
	  if ( i != N && uc_point[i] == uc_point[i+1] )
	    {
	      block_util += this_util;
	      block_cost += this_cost;
	    }
	  else
	    {
	      block_util = this_util;
	      block_cost = this_cost;
	    }
	  current_slope = block_util / block_cost;
#if DEBUGALG
	  if ( iteration <= 50 || (iteration%100) == 0)
	    {
	      printf("%d: util: %f, cost: %f, slope: %f.\n",
		     i, block_util, block_cost, current_slope );
	    }
#endif
	  if ( current_slope > best_slope )
	    {
	      best_slope = current_slope;
	      best_index = i;
	      best_cost = block_cost;
	      best_util = block_util;
	    }
	}
      if ( best_index < 0 )
	{
	  fprintf(stderr, "ALG_NEW: Couldn't find a positive slope!\n");
	  return NULL;
	}

      if( (used_cost+best_cost) > config->cost_to_use )
	cost_fraction = (config->cost_to_use - used_cost) / best_cost;
      
      used_cost += best_cost;
      used_util += best_util;

      ucp = uc_point[best_index];
      for ( i = best_index; i <= N; i++ )
	{
	  if ( uc_point[i] != ucp )
	    break;
#if DEBUGALG
	  printf("It: %4d.  Added to %3d (cost: %f, util: %f) slope: %f.\n",
		  iteration, i,
		 used_cost, used_util, best_slope );
#endif
	  utilities[i] = uc_point[i]->next->utility;
	  if ( cost_fraction < 1.0 )
	    {
	      utilities[i] =
		(uc_point[i]->next->utility * cost_fraction)
		+ (uc_point[i]->utility * (1.0 - cost_fraction));
	    }

	  uc_point[i] = ucp->next;
	}
    }
  printf("%d iterations.\n", iteration);

  assignment->guide_set = TRUE;
  for( i = 0; i <= N; i++ )
    {
      assignment->guide_utility[i] = utilities[i];
    }

  free( uc_point );
  free( utilities );

  return assignment;
}
