/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

int compare_uc_entry( const void *va, const void *vb )
{
  UC_ENTRY *a = (UC_ENTRY *) va;
  UC_ENTRY *b = (UC_ENTRY *) vb;
  if ( a->right_slope < b->right_slope )
    return -1;
  else
    return 1;
}

void insertion_sort( UC_ENTRY ** uc_array, int num )
{
  int i;
  int sort_pos;
  int best_pos;
  UC_ENTRY * temp;
  for( sort_pos = 0; sort_pos < num; sort_pos++ )
    {
      best_pos = sort_pos;
      for ( i = sort_pos + 1; i < num; i++ )
	{
	  if ( uc_array[i]->right_slope >
	       uc_array[best_pos]->right_slope )
	    best_pos = i;
	}
      temp = uc_array[best_pos];
      uc_array[best_pos] = uc_array[sort_pos];
      uc_array[sort_pos] = temp;
    }
}

int interface_array( int *costs, double *utilities, int *keeps, int num,
                     int total_packets, int received_packets,
		     int packet_length, double mean )
{
  CONFIG_DATA * config;
  UC_CURVE * uc_curve;
  LOSS_PROFILE * loss_profile;
  ASSIGNMENT * assignment;
  UC_ENTRY ** uc_array;
  long i;
  long source_length;
  long sum_cost;
  double sum_utility;

#if ULPSIM_RUSAGE
  struct rusage usage;
#endif
  struct timeval start_time;
  struct timeval elapsed_time;

  // Initialize the configuration data.
  config = malloc( sizeof( CONFIG_DATA ) );
  config->utility_type = UTILITY_MSE;
  config->cost_type = COST_BIT;
  config->symbol_size = 8;
  config->number_descriptions = total_packets;
  config->description_length = packet_length;
  config->number_symbols = config->number_descriptions
    * config->description_length;
  config->cost_to_use = config->number_symbols * config->symbol_size;

  // Allocate an array to hold the cost and utility information.
  uc_array = malloc( sizeof(UC_ENTRY *) * (num+1) );
  uc_array[0] = malloc( sizeof( UC_ENTRY ) );
  for( i = 0; i < num; i++ )
    {
      uc_array[i+1] = malloc( sizeof( UC_ENTRY ) );
      uc_array[i+1]->cost = costs[i];
      uc_array[i+1]->utility = utilities[i];
      uc_array[i+1]->index = i;
      uc_array[i+1]->right_slope = uc_array[i+1]->utility / uc_array[i+1]->cost;
      uc_array[i+1]->prev = NULL;
      uc_array[i+1]->next = NULL;
    }

#if DEBUGINTERFACE
  fprintf(stderr, "Before sort.\n");
  for( i = 1; i <= num; i++ )
    {
      fprintf( stderr, "%2ld: %5.1f %5.2f (%2d) %5.2f\n",
	       i,
	       uc_array[i]->cost,
	       uc_array[i]->utility, 
	       uc_array[i]->index,
	       uc_array[i]->right_slope );
    }
#endif

  // Sort the cost/utility information so important is at array pos 0.
#if 0
  qsort( uc_array+1, num, sizeof( UC_ENTRY * ), compare_uc_entry );
#else
  insertion_sort( uc_array+1, num );
#endif

#if DEBUGINTERFACE
  fprintf(stderr, "After sort.\n");
  for( i = 1; i <= num; i++ )
    {
      fprintf( stderr, "%2ld: %5.1f %5.2f (%2d) %5.2f\n",
	       i,
	       uc_array[i]->cost,
	       uc_array[i]->utility, 
	       uc_array[i]->index,
	       uc_array[i]->right_slope );
    }
#endif

  // We were passed costs and utilities for individual elements.
  // We need the cumulative values for the optimization.
  uc_array[0]->cost = 0;
  uc_array[0]->utility = 0;
  sum_cost = sum_utility = 0;
  for( i = 1; i <= num; i++ )
    {
      sum_cost += uc_array[i]->cost;
      sum_utility += uc_array[i]->utility;
      uc_array[i]->cost = sum_cost;
      uc_array[i]->utility = sum_utility;
    }

  for( i = 0; i <= num; i++ )
    {
      // Link the elements of the array in a linked list
      if ( i < num )
	uc_array[i]->next = uc_array[i+1];
      if ( i > 0 )
	uc_array[i]->prev = uc_array[i-1];

      // Make sure the slope entry is calculated.
      set_uc_slope( uc_array[i] );
    }

  uc_curve = alloc_uc_curve( );
  uc_curve->actual_head = uc_array[0];
  uc_curve->convex_head = copy_uc_entry_list( uc_array[0] );

  fprintf(stderr,"Creating PMF.\n");
  loss_profile = create_pmf_old_exponential( config->number_descriptions, mean );

  fprintf(stderr,"Allocating FEC.\n");

#if ULPSIM_RUSAGE
  getrusage( RUSAGE_SELF, &usage );
  elapsed_time = usage.ru_utime;
#else
  gettimeofday(&start_time,NULL);
#endif

  // Do the algorithm here.
  assignment = algorithm_new( uc_curve, loss_profile, config );

#if ULPSIM_RUSAGE
  getrusage( RUSAGE_SELF, &usage );
  elapsed_time.tv_sec = usage.ru_utime.tv_sec - elapsed_time.tv_sec;
  elapsed_time.tv_usec = usage.ru_utime.tv_usec - elapsed_time.tv_usec;
#else
  gettimeofday(&elapsed_time,NULL);
  elapsed_time.tv_sec -= start_time.tv_sec;
  elapsed_time.tv_usec -= start_time.tv_usec;
#endif

  if ( elapsed_time.tv_usec < 0 )
      {
	  elapsed_time.tv_sec--;
	  elapsed_time.tv_usec += 1000000;
      }
  fprintf(stderr, "Algorithm took %f seconds.\n",
          (double) elapsed_time.tv_sec
          + (double) elapsed_time.tv_usec / 1000000.0 );

  // Find the RS code strengths for each stream.
  if ( assignment->guide_set &&
       !assignment->actual_set &&
       !assignment->code_set )
    utilities_to_code_strength( assignment, uc_curve, config );

#if 1
  // Print out the codes assigned.
  fprintf(stderr, "Codes assigned:\n");
  for ( i = 0; i < packet_length; i++ )
    {
      fprintf(stderr, "%3ld %3d\n",
	      i, assignment->code_strength[i] );
    }
#endif

  // Calculate how much of the source is decodeable.
  for ( source_length = 0, i = 0; i < packet_length; i++ )
    {
      if ( assignment->code_strength[i] <= received_packets )
	source_length += assignment->code_strength[i];
    }
#if DEBUGINTERFACE
  fprintf(stderr, "Have %ld symbols and %ld bits.\n",
	  source_length, source_length * config->symbol_size );
#endif
  source_length *= config->symbol_size;

  // Set the keep array to 0 for everything.
  for ( i = 0; i < num; i++ )
    keeps[i] = 0;

  // Set the keep array to 1 for decodable elements.
  for ( i = 1; i <= num; i++ )
    {
      // If we don't have enough source data left, break.
      if ( source_length < uc_array[i]->cost )
	{
#if DEBUGINTERFACE
	  fprintf(stderr, "Not enough source, breaking.\n" );
#endif
	  break;
	}

      // Set that data to be kept.
      keeps[uc_array[i]->index] = 1;
    }

  return 0;
}
