/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

void check_block( void * block )
{
  if ( block )
    return;

  perror("Unable to malloc:");
  abort();
}

ASSIGNMENT * alloc_assignment( CONFIG_DATA * config )
{
  ASSIGNMENT * block;
  int i;
  int len;

  len = sizeof( ASSIGNMENT );
  block = malloc( len );
  check_block( block );

  block->code_set = FALSE;
  block->guide_set = FALSE;
  block->actual_set = FALSE;

  len = sizeof( int ) * config->description_length;
  block->code_strength = malloc( len );
  check_block( block->code_strength );

  len = sizeof( double ) * (config->number_descriptions+1);
  block->guide_utility = malloc( len );
  check_block( block->guide_utility );

  len = sizeof( double ) * (config->number_descriptions+1);
  block->actual_utility = malloc( len );
  check_block( block->actual_utility );

  return block;
}

void free_assignment( ASSIGNMENT * block )
{
  free( block->code_strength );
  free( block->guide_utility );
  free( block->actual_utility );
  free( block );
}


LOSS_PROFILE * alloc_loss_profile( int N )
{
  LOSS_PROFILE * block;
  int i;
  block = malloc( sizeof( LOSS_PROFILE ) );
  block->number_descriptions = N;
  block->loss_PMF = malloc( sizeof( double ) * (N+1) );
  block->receive_PMF = malloc( sizeof( double ) * (N+1) );
  for( i = 0; i <= N; i++ )
    block->loss_PMF[i] = block->receive_PMF[i] = 0.0;
  return block;
}

void free_loss_profile( LOSS_PROFILE * block )
{
  free( block->loss_PMF );
  free( block->receive_PMF );
  free( block );
}


UC_CURVE * alloc_uc_curve()
{
  UC_CURVE * block;
  block = malloc( sizeof( UC_CURVE ) );
  block->actual_head = NULL;
  block->convex_head = NULL;
  return block;
}

void free_uc_curve( UC_CURVE * block )
{
  UC_ENTRY *e;
  while( (e = block->actual_head) )
    {
      block->actual_head = e->next;
      free_uc_entry( e );
    }
  while( (e = block->convex_head) )
    {
      block->convex_head = e->next;
      free_uc_entry( e );
    }
  free( block );
}


UC_ENTRY * alloc_uc_entry()
{
  UC_ENTRY * block;
  block = malloc( sizeof( UC_ENTRY ) );
  block->prev = NULL;
  block->next = NULL;
  return block;
}

void free_uc_entry( UC_ENTRY * block )
{
  free( block );
}

