/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

double factorial_ln( int n )
{
  static double memoized[1000];
  int i;
  double value;

  if ( n < 0 )
    {
      fprintf(stderr,"Error: Negative value to factorial_ln\n");
      exit(1);
    }

  if ( n <= 1 )
    return 0.0;

  if ( n < 1000 && memoized[n] )
    return memoized[n];

  value = 0;
  for ( i = n; i > 1; i-- )
    {
      value += log( (double) i );
    }

  if ( n < 1000 )
    memoized[n] = value;

  return value;
}

double n_choose_k_ln( int n, int k )
{
  return factorial_ln( n ) - (factorial_ln( k ) + factorial_ln( n - k ));
}

double probability_ln( int n, int k, double p )
{
  return n_choose_k_ln( n, k ) + (double) k*log(p) + (double) (n-k)*log(1.0 - p);
}

LOSS_PROFILE * create_pmf_binomial( int N, double mean )
{
    int i;
    double total, calc_mean;
    LOSS_PROFILE *profile;

    profile = alloc_loss_profile( N );
    
    for( i = 0; i <= N; i++ )
      profile->loss_PMF[i] = exp(probability_ln( N, i, mean ));

    total = 0.0;
    calc_mean = 0.0;
    for( i = 0; i <= N; i++ )
      {
	profile->receive_PMF[N-i] = profile->loss_PMF[i];
	total += profile->loss_PMF[i];
	calc_mean += (double) i * profile->loss_PMF[i] / (double) N;
      }

    printf("Probabilities:\n");
    for( i = 0; i <= N; i++ )
      {
	printf("%3d - %.10f\n", i, profile->loss_PMF[i] );
      }

    printf("Expected loss: %f.  Total: %f\n",
	   calc_mean, total );
    return profile;
}
