/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"


LOSS_PROFILE * create_pmf_exponential( int N, double mean )
{
    int i,j;
    int start, stop;
    double total_tries;
    long iterations=0;
    double fraction;
    double my_exp;
    double last_exp;
    double high_exp;
    double low_exp;
    double prob_sum;
    LOSS_PROFILE *profile;
    double *lost;

    lost = calloc( N+1, sizeof( double ) );

    profile = alloc_loss_profile( N );

    for( i = 0; i <= N; i++ )
      lost[i] = 0.0;
    
    start = 0;
    stop = N;

    low_exp = 0.0;
    high_exp = 3000.0;
    my_exp = 10.0;
    last_exp = 1.0;

frac_start:
    if ( iterations++ > 10000 )
      {
	fprintf(stderr, "Error: unable to match requested loss rate.\n");
	exit(2);
      }

    for( j = 0; j <= stop; j++ )
      lost[j] = exp(0.0 - my_exp*(double)j/(double)stop);

    total_tries = 0.0;
    // Sum in reverse to minimize roundoff.
    for( j = stop; j >= 0; j-- )
      total_tries += lost[j];

#if 0
    for( j = 0; j <= stop; j++ )
      {
	total_tries -= lost[stop];
	lost[j] -= lost[stop];
      }
#endif

    fraction = 0.0;
    for( j = N; j >= 0; j-- )
      fraction += (double) lost[j] * (double) j / (double) N;

    fraction /= total_tries;

#if DEBUGPMF
    printf("Test_loss: %f yields %f\n", my_exp, fraction );
#endif
#define ACCURACY 0.00000001

    if ( fraction > (mean + ACCURACY) )
      {
	low_exp = my_exp;
	my_exp = low_exp + high_exp;
	my_exp /= 2;
	if ( my_exp != last_exp )
	  {
	    last_exp = my_exp;
	    goto frac_start;
	  }	    
      }
    if ( fraction < (mean - ACCURACY) )
      {
	high_exp = my_exp;
	my_exp = low_exp + high_exp;
	my_exp /= 2;
	if ( my_exp != last_exp )
	  {
	    last_exp = my_exp;
	    goto frac_start;
	  }	    
      }    
    printf("Expected loss for exponential PMF: %f\n", fraction );

#if DEBUGPMF
    printf("Counts:\n");
    for( j = 0; j <= N; j++ )
      {
	printf("%3d - %12.10f\n", j, lost[j]);
      }
#endif

    for( j = 0; j <= N; j++ )
      {
	profile->loss_PMF[j] = lost[j] / total_tries;
	profile->receive_PMF[N-j] = lost[j] / total_tries;
      }
#if DEBUGPMF
    printf("Probabilities:\n");
#endif
    prob_sum = 0.0;
    for( j = 0; j <= N; j++ )
      {
#if DEBUGPMF
	printf("%3d - %.10f\n", j, profile->loss_PMF[j] );
#endif
	prob_sum += j * profile->loss_PMF[j];
      }
    free( lost );
    return profile;
}
