/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

UC_CURVE *read_ulp_data_file( CONFIG_DATA * config )
{
  FILE *in;
  UC_CURVE *uc_curve;
  UC_ENTRY *uc_head = NULL;
  UC_ENTRY *current, *prev;
  char buf[MAX_STRING_LENGTH];
  double d1, d2, d3, d4;
  double rate, distortion, psnr;
  int count;
  int i;

  count = 0;
  in = fopen( config->fileName, "r" );
  if ( !in )
    {
      fprintf(stderr, "Unable to open ulp data file: [%s]\n",
	      config->fileName );
      exit(1);
      return NULL;
    }

  prev = current = NULL;

  for( count = 1;
       !feof( in ) && fgets( buf, MAX_STRING_LENGTH-1, in );
       count++ )
    {
      // If comment, continue.
      if ( *buf == '#' )
	continue;

      prev = current;
      current = alloc_uc_entry();

      if ( config->fileFormat == FORMAT_SPIHT )
	{
	  if ( sscanf( buf, "%lf %lf %lf", &d1, &d2, &d3 ) != 3 )
	    {
	      fprintf(stderr, "Error, line %d in %s invalid for format %d:\n[%s]\n",
		      count, config->fileName, config->fileFormat, buf );
	      return NULL;
	    }

	  current->cost = d1
	    * config->image_height
	    * config->image_width
	    * config->image_frames;

	  if ( config->utility_type == UTILITY_MSE )
	    current->utility = 0 - d2;
	  else if ( config->utility_type == UTILITY_PSNR )
	    current->utility = d3;
	  else
	    {
	      fprintf(stderr, "Error, Unknown utility type: %d.\n",
		      config->utility_type );
	      return NULL;
	    }
	}
      else if ( config->fileFormat == FORMAT_BITCOST_UTILITY )
	{
	  if ( sscanf( buf, "%lf %lf", &d1, &d2 ) != 2 )
	    {
	      fprintf(stderr, "Error, line %d in %s invalid for format %d:\n[%s]\n",
		      count, config->fileName, config->fileFormat, buf );
	      return NULL;
	    }

	  current->cost = d1;
	  current->utility = d2;
	}
      else if ( config->fileFormat == FORMAT_BYTECOST_UTILITY )
	{
	  if ( sscanf( buf, "%lf %lf", &d1, &d2 ) != 2 )
	    {
	      fprintf(stderr, "Error, line %d in %s invalid for format %d:\n[%s]\n",
		      count, config->fileName, config->fileFormat, buf );
	      return NULL;
	    }

	  current->cost = d1 * 8;
	  current->utility = d2;
	}
      else
	{
	  fprintf(stderr, "Error, Unknown format type: %d.\n",
		  config->fileFormat );
	  return NULL;
	}

      set_uc_slope( current );

      if ( prev == NULL )
	{
	  uc_head = current;
	}
      else
	{
	  prev->next = current;
	  current->prev = prev;
	  set_uc_slope( prev );
	}
    }

  uc_curve = alloc_uc_curve( );
  uc_curve->actual_head = uc_head;
  uc_curve->convex_head = copy_uc_entry_list( uc_head );
  convexify( uc_curve, config );

  return uc_curve;
}
