/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

int utilities_to_code_strength( ASSIGNMENT * assignment,
				 UC_CURVE * uc_curve,
				 CONFIG_DATA * config )
{
  int total_symbols_used = 0;
  int source_symbols_used = 0;

  double current_util;
  int max_symbols = config->number_symbols;
  int N = config->number_descriptions;
  int i;
  int count;
  UC_ENTRY * uc_entry = uc_curve->actual_head;
  
#ifdef DEBUGGUIDE
  for( i = 0; i <= N; i++ )
    {
      printf("Rectify: Guide Utility: %4d %7.4f\n",
	     i, assignment->guide_utility[i] );
    }
#endif

  current_util = uc_curve->actual_head->utility;
  assignment->actual_utility[0] = current_util;

  count = 0;
  for( i = 1; i <= N; i++ )
    {
      if ( current_util >= assignment->guide_utility[i]
	   || (!config->matching && total_symbols_used >= max_symbols) )
	{
#ifdef DEBUGGUIDE
	  printf("Assigning %3d: %f -> %f\n",
		 i, current_util, assignment->guide_utility[i] );
#endif
	  assignment->actual_utility[i] = current_util;
	  continue;
	}
      total_symbols_used += N;
      source_symbols_used += i;
      if ( config->matching == 0 )
	assignment->code_strength[count] = i;
      count++;
      if ( uc_entry->next == NULL )
	{
	  if ( config->matching == 2 )
	    return count+1;

	  fprintf( stderr,
		   "Rectify.c::utilities_to_code_strengths.  Guide utility (%f) is greater than the maximum utility (%f) in source utility-cost curve.\n",
		 assignment->guide_utility[i],
		 uc_entry->utility );
	  exit(3);
	}
      current_util = get_utility_memo( source_symbols_used
				       * config->symbol_size,
				       &uc_entry,
				       config );
      i--;
    }
  assignment->code_set = TRUE;
  assignment->actual_set = TRUE;
  if ( config->matching <= 1 )
    printf("Used %d symbols in each description (%d source, %d total).\n",
	   count, source_symbols_used, total_symbols_used);
  return count;
}

// Works, but slow.
double get_utility( int bits, UC_ENTRY *uc_entry, CONFIG_DATA * config )
{
  return get_utility_memo( bits, &uc_entry, config );
}

// Same as get_utility but takes a UC_ENTRY ** and updates it to
// save the last vertex on the utility-cost curve so that the next
// call (which is guaranteed to want that point or a point to the
// right of it) can access that point much more quickly.
double get_utility_memo( int bits,
			 UC_ENTRY ** uc_entry_p,
			 CONFIG_DATA * config )
{
  UC_ENTRY *uc_entry = *uc_entry_p;

  if ( uc_entry->cost == bits || uc_entry->next == NULL )
    return uc_entry->utility;

  if ( uc_entry->cost < bits )
    {
      double fraction;
      if ( uc_entry->next->cost <= bits )
	{
	  *uc_entry_p = uc_entry->next;
	  return get_utility_memo( bits, uc_entry_p, config );
	}

      fraction = (double) (bits - uc_entry->cost)
	/ (double) (uc_entry->next->cost - uc_entry->cost);
      
      return fraction * uc_entry->next->utility
	+ (1.0-fraction) * uc_entry->utility;
    }

  fprintf( stderr, "Error: rectify: get_utility_memo().  Cost was > bits.\n" );
  exit(1);
}
