/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

int main(int argc, char **argv)
{
  CONFIG_DATA * config;
  UC_CURVE * uc_curve;
  ASSIGNMENT * original_assignment;
  ASSIGNMENT * assignment;
  double bonus = 0.0;
  int i;
  int count;
  int rate_constrained = 0;

  if ( 1 )
    {
      char buf[MAX_STRING_LENGTH];
      config = malloc( sizeof( CONFIG_DATA ) );
      
      printf("Enter data file name> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      if ( strlen(buf) > 0 && buf[strlen(buf)-1] == '\n' )
	buf[strlen(buf)-1] = 0;
      config->fileName = strdup(buf);
      printf("Using file name [%s].\n\n", buf );

    choose_format:
      printf("Choose a file format:\n");
      printf("1 - SPIHT (first column bit rate, second mse, third psnr).\n");
      printf("2 - First column cost in bits, second column utility.\n");
      printf("3 - First column cost in bytes, second column utility.\n");
      printf("> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->fileFormat = atoi(buf);
      if ( config->fileFormat <= 0 || config->fileFormat > 3 )
	{ printf("Invalid file format.\n\n"); goto choose_format; }
      printf("Using format [%d].\n\n", config->fileFormat );

      // SPIHT defaults
      if ( config->fileFormat == FORMAT_SPIHT )
	{
	  config->cost_type = COST_RATE;
	  config->utility_type = UTILITY_PSNR;

	choose_psnr_mse:
	  printf("Do you want to use PSNR or MSE for utility [p/m]?\n");
	  printf("> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  if ( *buf == 'p' || *buf == 'P' )
	    config->utility_type = UTILITY_PSNR;
	  else if ( *buf == 'm' || *buf == 'M' )
	    {
	      config->utility_type = UTILITY_MSE;
	      printf("*** NOTE: The simulator assumes utilities increase with rate, so\n");
	      printf("          MSE's will be reported as negative numbers.  Future\n");
	      printf("          versions will fix that.\n");
	    }
	  else
	    goto choose_psnr_mse;

	choose_image_width:
	  printf("Enter the image width> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  config->image_width = atoi(buf);
	  if ( config->image_width <= 0 )
	    { printf("Invalid number.\n\n"); goto choose_image_width; }
	  printf("Using image width of %d.\n\n",
		 config->image_width);

	choose_image_height:
	  printf("Enter the image height> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  config->image_height = atoi(buf);
	  if ( config->image_height <= 0 )
	    { printf("Invalid number.\n\n"); goto choose_image_height; }
	  printf("Using image height of %d.\n\n",
		 config->image_height);

	  config->image_frames = 1;
	  config->image_bits = 8;
	}
	  
    choose_number_descriptions:
      printf("Enter the number of packets/descriptions> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->number_descriptions = atoi(buf);
      if ( config->number_descriptions <= 0 )
	{ printf("Invalid number.\n\n"); goto choose_number_descriptions; }
      printf("Using %d packets/descriptions.\n\n",
	     config->number_descriptions);

    choose_description_length:
      printf("This program can report the number of bytes necessary to\n");
      printf("achieve your target utilities or it can match the utilities\n");
      printf("as closely as possible for a given total cost.\n");
      printf("If you want to achieve the target utilities, enter 0.\n");
      printf("Otherwise, enter the length of each packet/description in bytes.\n");
      printf("> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->description_length = atoi(buf);
      if ( config->description_length <= 0 )
	config->description_length = 1;
      else
	{
	  rate_constrained = 1;
	}

      // Derived values.
      config->matching = 1;
      config->symbol_size = 8;
      config->number_symbols = config->number_descriptions
	* config->description_length;
      config->cost_to_use = config->number_symbols * config->symbol_size;
    }
  
  fprintf(stderr,"Reading data file.\n");
  uc_curve = read_ulp_data_file( config );

  assignment = alloc_assignment( config );
  original_assignment = alloc_assignment( config );

  for( i = 1; i <= config->number_descriptions; i++ )
    {
      char buf[MAX_STRING_LENGTH];
      printf("Utility when %d packets/descriptions are received (%d lost)> ",
	     i, config->number_descriptions - i);
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      original_assignment->guide_utility[i] = atof(buf);
      printf("Target utility for %d descriptions: %f.\n",
	     i, original_assignment->guide_utility[i]);
    }
  printf("\n");

  if ( rate_constrained )
    {
      double high_bonus, low_bonus;
      double best_bonus = 0.0;
      int iteration = 0;
      count = -1;

      /* Add up to the difference between the highest target 
	 and the lowest target. */
      high_bonus = original_assignment->guide_utility[config->number_descriptions]
	- original_assignment->guide_utility[1];
      
      /* Subtract up to the difference between the highest and lowest. */
      low_bonus = original_assignment->guide_utility[1]
	- original_assignment->guide_utility[config->number_descriptions];
      
      config->matching = 2;
      for( iteration = 0;
	   iteration < 50;
	   iteration++ )
	{
	  for( i = 1; i <= config->number_descriptions; i++ )
	    assignment->guide_utility[i]
	      = original_assignment->guide_utility[i] + bonus;
	  
	  count = utilities_to_code_strength( assignment, uc_curve, config );
	  
	  if ( count <= config->description_length )
	    {
	      low_bonus = bonus;
	      best_bonus = bonus;
	    }
	  else
	    high_bonus = bonus;

	  bonus = low_bonus + high_bonus;
	  bonus /= 2.0;

	  if ( count == config->description_length )
	    iteration += 5;
	}
      bonus = best_bonus;
      if ( bonus >= 0.0 )
	printf("Exceeding targets by %f.\n", bonus );
      else
	printf("Only able to get within %f of targets.\n", bonus );
      config->matching = 1;
    }

  for( i = 1; i <= config->number_descriptions; i++ )
    assignment->guide_utility[i] = original_assignment->guide_utility[i] + bonus;
  
  count = utilities_to_code_strength( assignment, uc_curve, config );

  for( i = 1; i <= config->number_descriptions; i++ )
    printf("%3d descriptions, target utility: %10.5f (%10.5f achieved) %s%.5f.\n",
	   i,
	   original_assignment->guide_utility[i],
	   assignment->actual_utility[i],
	   assignment->actual_utility[i] >= original_assignment->guide_utility[i]
	   ? "+" : "",
	   assignment->actual_utility[i] - original_assignment->guide_utility[i]
	   );

  printf("%d bytes used in each of %d packets for a total of %d bytes.\n",
	 count,
	 config->number_descriptions,
	 count * config->number_descriptions );

  return 0;
}
