/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

int main(int argc, char **argv)
{
  CONFIG_DATA * config;
  UC_CURVE * uc_curve;
  LOSS_PROFILE * loss_profile;
  ASSIGNMENT * assignment;
#if ULPSIM_RUSAGE
  struct rusage usage;
#endif
  struct timeval start_time;
  struct timeval elapsed_time;

  double mean = 0.20;
  double redundancy = 0.0;
  int loss_model = 0;
  int iteration = 0;

  if ( 1 )
    {
      char buf[MAX_STRING_LENGTH];
      config = malloc( sizeof( CONFIG_DATA ) );
      
      config->matching = 0;

      printf("Enter data file name> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      if ( strlen(buf) > 0 && buf[strlen(buf)-1] == '\n' )
	buf[strlen(buf)-1] = 0;
      config->fileName = strdup(buf);
      printf("Using file name [%s].\n\n", buf );

    choose_format:
      printf("Choose a file format:\n");
      printf("1 - SPIHT (first column bit rate, second mse, third psnr).\n");
      printf("2 - First column cost in bits, second column utility.\n");
      printf("3 - First column cost in bytes, second column utility.\n");
      printf("> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->fileFormat = atoi(buf);
      if ( config->fileFormat <= 0 || config->fileFormat > 3 )
	{ printf("Invalid file format.\n\n"); goto choose_format; }
      printf("Using format [%d].\n\n", config->fileFormat );

      // SPIHT defaults
      if ( config->fileFormat == FORMAT_SPIHT )
	{
	  config->cost_type = COST_RATE;
	  config->utility_type = UTILITY_PSNR;

	choose_psnr_mse:
	  printf("Do you want to use PSNR or MSE for utility [p/m]?\n");
	  printf("> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  if ( *buf == 'p' || *buf == 'P' )
	    config->utility_type = UTILITY_PSNR;
	  else if ( *buf == 'm' || *buf == 'M' )
	    {
	      config->utility_type = UTILITY_MSE;
	      printf("*** NOTE: The simulator assumes utilities increase with rate, so\n");
	      printf("          MSE's will be reported as negative numbers.  Future\n");
	      printf("          versions will fix that.\n");
	    }
	  else
	    goto choose_psnr_mse;

	choose_image_width:
	  printf("Enter the image width> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  config->image_width = atoi(buf);
	  if ( config->image_width <= 0 )
	    { printf("Invalid number.\n\n"); goto choose_image_width; }
	  printf("Using image width of %d.\n\n",
		 config->image_width);

	choose_image_height:
	  printf("Enter the image height> ");
	  fgets( buf, MAX_STRING_LENGTH-1, stdin );
	  config->image_height = atoi(buf);
	  if ( config->image_height <= 0 )
	    { printf("Invalid number.\n\n"); goto choose_image_height; }
	  printf("Using image height of %d.\n\n",
		 config->image_height);

	  config->image_frames = 1;
	  config->image_bits = 8;
	}
	  
    choose_number_descriptions:
      printf("Enter the number of packets/descriptions> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->number_descriptions = atoi(buf);
      if ( config->number_descriptions <= 0 )
	{ printf("Invalid number.\n\n"); goto choose_number_descriptions; }
      printf("Using %d packets/descriptions.\n\n",
	     config->number_descriptions);

    choose_description_length:
      printf("Enter the length of each packet/description in symbols> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->description_length = atoi(buf);
      if ( config->description_length <= 0 )
	{ printf("Invalid number.\n\n"); goto choose_description_length; }
      printf("Using %d symbols in each description.\n\n",
	     config->description_length);

    choose_symbol_size:
      printf("Enter the number of bits in each symbol (8 for bytes)> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      config->symbol_size = atoi(buf);
      if ( config->symbol_size <= 0 )
	{ printf("Invalid number.\n\n"); goto choose_symbol_size; }
      printf("Using symbols with %d bits in each.\n\n",
	     config->symbol_size);

    choose_loss_model:
      printf("Pick a loss model:\n");
      printf("1 - Exponential.\n");
      printf("2 - Binomial.\n");
      printf("3 - \"Uniform\".\n");
      printf("> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      loss_model = atoi(buf);
      if ( loss_model <= 0 )
	{ printf("Invalid loss model.\n\n"); goto choose_loss_model; }
      printf("Using loss model %d.\n\n",
	     loss_model);

    choose_derived:
      printf("The simulator can optimize for a given set of channel/network conditions in the\n");
      printf("form of a mean loss rate or (for research purposes) it can achieve a given\n");
      printf("fraction of redundant information by finding a loss rate that results in\n");
      printf("that redundancy fraction.\n");
      printf("Select an option:\n");
      printf("(o) : Optimize for channel/network conditions.\n");
      printf("(r) : Redundancy fraction.?\n");
      printf("> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      if ( *buf == 'o' || *buf == 'O' )
	goto choose_mean;
      if ( *buf == 'r' || *buf == 'R' )
	goto choose_redundancy;
      printf("Invalid choice.  Enter 'c' or 'r'.\n\n");
      goto choose_derived;

    choose_mean:
      printf("Enter the mean loss rate (range 0 to 1)> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      mean = atof( buf );
      if ( mean < 0 || mean >= 1 )
	{ printf("Invalid mean loss rate.\n\n"); goto choose_mean; }
      printf("Using loss rate %f.\n\n", mean );
      goto derive_values;

    choose_redundancy:
      if ( loss_model == 3 )
	{
	  printf("Error: You cannot specify a redundancy fraction with the uniform loss model.\n");
	  goto choose_loss_model;
	}
      printf("Enter the redundancy fraction (range 0 to 1)> ");
      fgets( buf, MAX_STRING_LENGTH-1, stdin );
      redundancy = atof( buf );
      if ( redundancy < 0 || redundancy >= 1 )
	{ printf("Invalid redundancy fraction.\n\n"); goto choose_redundancy; }
      printf("Using redundancy fraction %f.\n\n", redundancy );
      goto derive_values;

    derive_values:
      // Derived values.
      config->number_symbols = config->number_descriptions
	* config->description_length;
      config->cost_to_use = config->number_symbols * config->symbol_size;
    }
  


  fprintf(stderr,"Reading data file.\n");
  uc_curve = read_ulp_data_file( config );

 do_create_pmf:
  iteration++;
  fprintf(stderr,"Creating PMF.\n");
  if ( loss_model == 1 )
    loss_profile = create_pmf_exponential( config->number_descriptions,
					       mean );
  else if ( loss_model == 2 )
    loss_profile = create_pmf_binomial( config->number_descriptions, mean );
  else if ( loss_model == 3 )
    loss_profile = create_pmf_uniform( config->number_descriptions );
  else
    { printf("Unknown loss model: %d\n", loss_model); exit(1); }

  fprintf(stderr,"Allocating FEC.\n");

#if ULPSIM_RUSAGE
  getrusage( RUSAGE_SELF, &usage );
  elapsed_time = usage.ru_utime;
#else
  gettimeofday(&start_time,NULL);
#endif

  assignment = algorithm_new( uc_curve, loss_profile, config );

#if ULPSIM_RUSAGE
  getrusage( RUSAGE_SELF, &usage );
  elapsed_time.tv_sec = usage.ru_utime.tv_sec - elapsed_time.tv_sec;
  elapsed_time.tv_usec = usage.ru_utime.tv_usec - elapsed_time.tv_usec;
#else
  gettimeofday(&elapsed_time,NULL);
  elapsed_time.tv_sec -= start_time.tv_sec;
  elapsed_time.tv_usec -= start_time.tv_usec;
#endif

  if ( elapsed_time.tv_usec < 0 )
      {
	  elapsed_time.tv_sec--;
	  elapsed_time.tv_usec += 1000000;
      }
  fprintf(stderr, "Algorithm took %f seconds.\n",
          (double) elapsed_time.tv_sec
          + (double) elapsed_time.tv_usec / 1000000.0 );

  if ( assignment->guide_set &&
       !assignment->actual_set &&
       !assignment->code_set )
    utilities_to_code_strength( assignment, uc_curve, config );

  
  if ( redundancy > 0.0 )
    {
      static double high_mean=1.0;
      static double low_mean=0.0;
      int source_length = 0;
      double current_redundancy;
      int i;

      for( i = 0; i < config->description_length; i++ )
	source_length += assignment->code_strength[i];

      current_redundancy = 1.0 - ((double)source_length
				  / (double) config->number_symbols);

      printf("Iteration %3d: Found redundancy %f/%f at loss rate %f.\n",
	     iteration, current_redundancy, redundancy, mean );
      if ( iteration >= 50 )
	{
	  printf("Couldn't exactly achieve redundancy rate.  Close enough.\n");
	}
      else if ( current_redundancy < redundancy )
	{
	  low_mean = mean;
	  mean = (low_mean + high_mean) / 2;
	  goto do_create_pmf;
	}
      else if ( current_redundancy > redundancy )
	{
	  high_mean = mean;
	  mean = (low_mean + high_mean) / 2;
	  goto do_create_pmf;
	}
      /* Found the redundancy rate. */
      printf("Achieved redundancy of %f using loss rate %f.\n",
	     current_redundancy, mean );
    }

  if ( 1 )
    {
      int i;
      for( i = 0; i < config->description_length; i++ )
	printf("Code %3d is (%3d, %3d).\n",
	       i, config->number_descriptions,
	       assignment->code_strength[i]);
    }

  if ( 1 )
    print_assignment( assignment, loss_profile, config );

  /* Not sure if it's still valid.  old code.
    write_matlab_plot( assignment, loss_profile, "sim.mat", mean );
  */
  return 0;
}
