/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

void set_uc_slope( UC_ENTRY * current )
{
  // No benefit past the end of the data file.
  if ( current->next == NULL )
    current->right_slope = -1000000000.0;
  else
    current->right_slope = (current->next->utility - current->utility) /
      (current->next->cost - current->cost);
}


UC_ENTRY * copy_uc_entry_list( UC_ENTRY * actual_head )
{
  UC_ENTRY *prev, *current, *next;
  UC_ENTRY *head;
  UC_ENTRY *new_node;

  head = alloc_uc_entry();

  head->cost = actual_head->cost;
  head->utility = actual_head->utility;
  head->right_slope = actual_head->right_slope;
  head->prev = NULL;
  head->next = NULL;

  prev = head;

  current = actual_head->next;
  while( current )
    {
      new_node = alloc_uc_entry();
      new_node->cost = current->cost;
      new_node->utility = current->utility;
      new_node->right_slope = current->right_slope;
      new_node->next = NULL;
      new_node->prev = prev;
      prev->next = new_node;
      prev = new_node;

      current = current->next;
    }
  return head;
}

void convexify( UC_CURVE * uc_curve, CONFIG_DATA * config )
{
  UC_ENTRY *prev, *current, *next;
  UC_ENTRY *head;

  current = uc_curve->convex_head;
  while ( current->next )
    {
      prev = current->prev;
      next = current->next;

      // On the left edge, continue;
      if ( prev == NULL )
	{
	  current = next;
	  continue;
	}

      // If the slope decreases, we're fine.
      if ( prev->right_slope > current->right_slope )
	{
	  current = next;
	  continue;
	}

      // Now we have to splice out the current node.
      prev->next = next;
      next->prev = prev;

      set_uc_slope( prev );

      current->next = NULL;
      current->prev = NULL;
      // ALEXDEBUG
      //      free_uc_entry( current );

      current = prev;
    }
  return;
}

void print_assignment( ASSIGNMENT * assignment,
		       LOSS_PROFILE * profile,
		       CONFIG_DATA * config )
{
  int i;
  double expected_utility;
  expected_utility = 0;
  for( i = 0; i <= profile->number_descriptions; i++ )
    {
      printf("Received %3d: %6.4f utility.\n",
	     i, assignment->actual_utility[i]);
      expected_utility += assignment->actual_utility[i]
	* profile->receive_PMF[i];
    }
  printf("Expected utility: %6.4f.\n", expected_utility );
}


/* This function is no longer supported */
void write_matlab_plot( ASSIGNMENT * assignment,
			LOSS_PROFILE * profile,
			char * file_name,
			double mean
			)
{
  int i;
  double expected_utility;
  FILE *out;
  char buf[10000];

  sprintf( buf, "%s", file_name );
  out = fopen(buf, "w");

  fprintf(out, "ULPpsnrs = ([;\n");
  expected_utility = 0;
  for( i = 0; i <= profile->number_descriptions; i++ )
    {
      fprintf(out, "([%d %.8f]); ",
	      i, assignment->actual_utility[i]);
      if ( i % 4 == 3 )
	fprintf(out, "\n");
      expected_utility += assignment->actual_utility[i]
	* profile->receive_PMF[i];
    }
  fprintf(out, "]);\n");
  fprintf(out, "clf\n");
  fprintf(out, "set(gca,'FontSize', 14);\n");
  fprintf(out, "h1 = plot( ULPpsnrs(:,1), ULPpsnrs(:,2), 'b-' );\n");
  fprintf(out, "set(h1,'LineWidth',1);\n");
  fprintf(out, "legend([h1(1)], 'ULP Sim');\n");
  fprintf(out, "xlabel('Number of Descriptions Received');\n");
  fprintf(out, "ylabel('Expected PSNR at Receiver (dB)');\n");
  fprintf(out, "title('UNKNOWNIMAGE for %.2f mean binomial, %d packets, total rate UNKNOWN, expected PSNR %.2f dB, no arith coding');\n",
	  mean, profile->number_descriptions, expected_utility );
  fprintf(out, "grid on\n");
  fclose(out);

  printf("Expected utility: %6.4f.\n", expected_utility );
}


